==> iRRAM_2004_02.tar.gz: iRRAM_2004_02/src/GMP_int_ext.c <==
/*

GMP_int_ext.c -- extension to the GMP integer functions for the iRRAM library 
 
Copyright (C) 2001-2004 Norbert Mueller
 
This file is part of the iRRAM Library.
 
The iRRAM Library is free software; you can redistribute it and/or modify
it under the terms of the GNU Library General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.
 
The iRRAM Library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
License for more details.
 
You should have received a copy of the GNU Library General Public License
along with the iRRAM Library; see the file COPYING.LIB.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA. 
*/

#include "GMP_intrat.h"

#include <strings.h>

#ifndef DEBUG			// debugging information
#define DEBUG 0
#endif
#if DEBUG > 0
  int DEBUG_OP = 1;
  unsigned int op_count=0;
  int minshowop = DEBUG ;
#endif

//**************************************************************************/
//* Variables for counting free space */
//**************************************************************************/

#define MaxFreeVars 1000
static int_gmp_type gmp_FreeVarsi[MaxFreeVars];
static int gmp_FreeVarCounti=0L;
int int_gmp_var_count=0;


//**************************************************************************/
//* INITIALIZING FUNCTIONS */
//**************************************************************************/

int_gmp_type int_gmp_init()
{
	int_gmp_type z;
	if (gmp_FreeVarCounti>0) 
		{
		gmp_FreeVarCounti--;
		z = gmp_FreeVarsi[gmp_FreeVarCounti];
		}
	else
		{
		z = (int_gmp_type) malloc(sizeof(__mpz_struct));
		mpz_init(z);
		}
	int_gmp_var_count++;
	return z;
}





//*************************************************************************/
// delete no longer used GMP_integers, including deallocation
// added: 31.01.2001
// Last change: 31.01.2001
// Arguments: 1. GMP_integer
//*************************************************************************/

void int_gmp_free(int_gmp_type z)
{
  if (gmp_FreeVarCounti < MaxFreeVars)
  		{
  		gmp_FreeVarsi[gmp_FreeVarCounti]=z;
  		gmp_FreeVarCounti++;
  		}
  	else
  	{
  	mpz_clear(z);
  	free(z);
  	}
  int_gmp_var_count--;

  //mpz_clear(z);
}


//**************************************************************************/
// Getting size of GMP_integers
// Last change: 2004-02-11
// Arguments: 1. GMP_integer
// 		  return: (standard) integer
//**************************************************************************/

int int_gmp_size(int_gmp_type z) //****** Anzahl der Ziffern */
{
if (mpz_sgn(z) == 0 )
  return 0;
else 
  return mpz_sizeinbase(z,2);
}


//**************************************************************************/
// CONVERSION FUNCTIONS
//**************************************************************************/
// Converting standart integer (int) to MP_integer
// added: 01.05.2001
// Last change: 01.05.2001
// Arguments: 1. signed (standard) integer
//		  2. GMP_integer
//**************************************************************************/

void int_gmp_int2integer(const int i, int_gmp_type z)
{
 mpz_set_si(z,i);
 return;
}

//**************************************************************************/
// Converting standard double to MP_integer
// added: 01.05.2001
// Last change: 01.05.2001
// Arguments: 1. (standard) double
//		  2. GMP_integer
//**************************************************************************/

void int_gmp_double2int(const double d, int_gmp_type z)
{
 mpz_set_d(z,d);
 return;
}

//**************************************************************************/
// Converting string to MP_integer
// added: 01.05.2001
// Last change: 01.05.2001
// Arguments: 1. string
// 		  2. GMP_integer
// 		  3. Base (i.e. 10 for decimal)
//**************************************************************************/

void int_gmp_string2int(const char* s, int_gmp_type z, int b)
{
 mpz_set_str(z,s,b);
 return;
}

//**************************************************************************/
// Converting GMP_integer to int integer
// added: 01.05.2001
// Last change: 01.05.2001
// Arguments: 1. GMP_integer
// 		  2. int integer
//**************************************************************************/

int int_gmp_integer2int(const int_gmp_type i)
{
 return mpz_get_si(i);
}


//**************************************************************************/
// SIMPLE ARITHMETIC OPERATIONS
//**************************************************************************/
// adding GMP_integers
// calculates z = z1 + z2
// added: 31.01.2001
// Last change: 31.01.2001
// Arguments: 1. GMP_integer
// 		  2. GMP_integer
// 		  3. GMP_integer
//**************************************************************************/

void int_gmp_add(const int_gmp_type z1, const int_gmp_type z2, int_gmp_type z)
{
 mpz_add(z,z1,z2);
 return;
}

//**************************************************************************/
// adding GMP_integer and unsigned int
// calculates z = z1 + z2
// added: 18.06.2001
// Last change: 18.06.2001
// Arguments: 1. GMP_integer
// 		  2. unsigned int
// 		  3. GMP_integer
//**************************************************************************/

void int_gmp_add_ui(int_gmp_type z1, unsigned int z2, int_gmp_type z)
{
 mpz_add_ui(z,z1,z2);
 return;
}


//**************************************************************************/
// subtract GMP_integers
// calculates z = z1 - z2
// added: 01.02.2001
// Last change: 01.02.2001
// Arguments: 1. GMP_integer
// 		  2. GMP_integer
// 		  3. GMP_integer
//**************************************************************************/

void int_gmp_sub(int_gmp_type z1,int_gmp_type z2,int_gmp_type z)
{
  mpz_sub(z,z1,z2);
  return;
}

//**************************************************************************/
// subtract GMP_integer and unsigned int
// calculates z = z1 - z2
// added: 01.02.2001
// Last change: 01.02.2001
// Arguments: 1. GMP_integer
// 		  2. unsigned int
// 		  3. GMP_integer
//**************************************************************************/

void int_gmp_sub_ui(int_gmp_type z1,unsigned int z2,int_gmp_type z)
{
  mpz_sub_ui(z,z1,z2);
  return;
}


//**************************************************************************/
// multiplicate GMP-integers
// calculates z = z1 * z2
// added: 01.02.2001
// Last change: 01.02.2001
// Arguments: 1. GMP_integer
// 		  2. GMP_integer
// 		  3. GMP_integer
//**************************************************************************/

void int_gmp_mul(int_gmp_type z1, int_gmp_type z2, int_gmp_type z)
{
 mpz_mul(z,z1,z2);
 return;
}

//**************************************************************************/
// multiplicate GMP-integer by signed int
// calculates z = z1 * z2
// added: 01.02.2001
// Last change: 01.02.2001
// Arguments: 1. GMP_integer
// 		  2. signed int
// 		  3. GMP_integer
//**************************************************************************/

void int_gmp_mul_si(int_gmp_type z1, signed int z2, int_gmp_type z)
{
 mpz_mul_si(z,z1,z2);
 return;
}


//**************************************************************************/
// dividing GMP_integers
// calculates z = z1 / z2 == z1 DIV z2
// added: 01.02.2001
// Last change: 01.02.2001
// Arguments: 1. GMP_integer
// 		  2. GMP_integer
// 		  3. GMP_integer
//**************************************************************************/

void int_gmp_div(int_gmp_type z1, int_gmp_type z2, int_gmp_type z)
{
 mpz_tdiv_q(z,z1,z2); //**** truncated div ****
 return;
}

//**************************************************************************/
// dividing GMP_integer by unsigned int
// calculates z = z1 / z2 == z1 DIV z2
// added: 18.06.2001
// Last change: 18.06.2001
// Arguments: 1. GMP_integer
// 		  2. unsigned int
// 		  3. GMP_integer
//**************************************************************************/

void int_gmp_div_ui(int_gmp_type z1, unsigned int z2, int_gmp_type z)
{
 unsigned int dummy;
 dummy = mpz_tdiv_q_ui(z,z1,z2); //**** truncated div ****
 return;
}


//**************************************************************************/
// absolute value of integer in GMP
// returns z = |z1|
// added: 31.01.2001
// Last change: 31.01.2001
// Arguments: 1. GMP_integer
// 		  2. GMP_integer
//**************************************************************************/

void int_gmp_abs(int_gmp_type z1, int_gmp_type z)
{
 mpz_abs(z, z1);
 return;
}

//**************************************************************************/
// changes sign of GMP integer
// returns z = -z
// added: 18.06.2001
// Last change: 18.06.2001
// Arguments: 1. GMP_integer
// 		  2. GMP_integer
// Comment: used for operators with unsigned integers
//**************************************************************************/

void int_gmp_neg(int_gmp_type z1, int_gmp_type z)
{
  mpz_neg(z, z1);
  return;
}


//**************************************************************************/
// nth root of integer in GMP
// added: 07.02.2001
// Last change: 07.02.2001
// Comment: this root-function truncates the result to the integer part
// Arguments: 1. GMP_integer
// 		  2. unsigned int
// 		  3. GMP_integer
//**************************************************************************/

void int_gmp_root(int_gmp_type z1, unsigned int z2, int_gmp_type z)
{
 int temp;	
 temp=mpz_root(z, z1, z2);
}

//**************************************************************************/
// power function in GMP
// calculates z = z1 ^ z2
// added: 07.02.2001
// Last change: 07.02.2001
// Comment: base=MP exponent=int
// Arguments: 1. GMP_integer
// 		  2. unsigned integer
// 		  3. GMP_integer
//**************************************************************************/

void int_gmp_power_i(int_gmp_type z1, unsigned int z2, int_gmp_type z)
{
 mpz_pow_ui(z, z1, z2);
 return;
}

//**************************************************************************/
// power function in GMP
// calculates z = z1 ^ z2
// added: 07.02.2001
// Last change: 07.02.2001
// Comment: base=int exponent=int
// Arguments: 1. unsigned int
// 		  2. unsigned int
// 		  3. GMP_integer
//**************************************************************************/

void int_gmp_power_ii(unsigned int z1, unsigned int z2, int_gmp_type z)
{
 mpz_ui_pow_ui(z, z1, z2);
 return;
}

//**************************************************************************/
// faculty function in GMP
// calculates z = z1!
// added: 07.02.2001
// Last change: 07.02.2001
// Comment: base is int
// Arguments: 1. unsigned int
// 		  2. GMP_integer
//**************************************************************************/

void int_gmp_fac(unsigned int z1, int_gmp_type z)
{
 mpz_fac_ui(z, z1);
 return;
}

//**************************************************************************/
// modulo function in GMP
// calculates z = z1 % z2
// added: 07.02.2001
// Last change: 07.02.2001
// Arguments: 1. GMP_integer
// 		  2. GMP_integer
// 		  3. GMP_integer
//**************************************************************************/

void int_gmp_modulo(int_gmp_type z1, int_gmp_type z2, int_gmp_type z)
{
 mpz_mod(z, z1, z2);
 return;
}

//**************************************************************************/
// logarithm with base 2 
// calculates log(z)
// added: 31.03.2001
// Last change: 31.03.2001
// Arguments: 1. GMP_integer
// 		  return: integer 
//**************************************************************************/

int int_gmp_log(int_gmp_type z)
{
unsigned int r=0,w,n,m;
if (z->_mp_size==0) return GMP_min;
w=(z->_mp_d)[abs(z->_mp_size)-1];   //****** get the highest-valued limb *****/
n=w;
for (m=mp_bits_per_limb>>1; m>0; m=m>>1 ) //****** divide and conquer ******/
	{
      if (n>= (1<<m) )
		{
        	r+=m; n=n>>m;
      	}
    	}
if (w > (1<<r) || abs(z->_mp_size) >0) r++;    //******/
return r;
}

//**************************************************************************/

int int_gmp_log2(int_gmp_type z)
{
	unsigned int r=0,n,m;
	mp_limb_t temp;// could be int
	temp=mpz_getlimbn(z,1);
	n=temp;
	for (m=mp_bits_per_limb>>1; m>0; m=m>>1 ) //* kinda divide and conquer */
		{
      	if (n>= (1<<m) )
		{
        		r+=m; n=n>>m;
      	}
    	}
	if (temp > (1<<r) || abs(z->_mp_size) >0) r++; //* odd numbers */
	return r+(abs(z->_mp_size)-1)*32-1;
}



//**************************************************************************/
// writes integer to a string
// writes z to string with length w
// Arguments: 1. GMP_integer
// 		  2. int integer
//	return: char*
//**************************************************************************/


char* int_gmp_swritee(int_gmp_type z, int w)
{
  if (w<1)w=1;
  char *n, *s=malloc(w+1);
  int l;
  n=mpz_get_str(NULL, 10, z);
  l=strlen(n);
  if (l>w){
    strncpy(s,n,w);
    s[w-1]='*';
    s[w]='\0';
  }else{
    memset(s,' ',w-l);
    strncpy(&s[w-l],n,l);
  } 
  free(n); 
  return s;
}



//**************************************************************************/
// writes z to string with length calculated
// added: 18.07.2001
// Last change: 18.07.2001
// Arguments: 1. GMP_integer
// 	return: char*
//**************************************************************************/

char* int_gmp_sprintf(int_gmp_type z)
{
  return mpz_get_str(NULL,10,z);
}


//**************************************************************************/
// duplicate initialized integer in GMP
// copies z1 into already initialized GMP_integer z2
// added: 01.02.2001
// Last change: 01.02.2001
// Arguments: 1. GMP_integer
// 		  2. GMP_integer
//**************************************************************************/

void int_gmp_duplicate_wo_init(int_gmp_type z1, int_gmp_type z2)
{
 mpz_set(z2,z1);
 return;
}



//**************************************************************************/
// duplicate non-initialized integer in GMP
// copies z1 into non-initialized GMP_integer z2
// added: 01.02.2001
// Last change: 01.02.2001
// Arguments: 1. GMP_integer
// 		  2. GMP_integer
//**************************************************************************/

void int_gmp_duplicate_w_init(int_gmp_type z1, int_gmp_type *z2)
{
  *z2=int_gmp_init();
  mpz_set(*z2,z1);
}


//**************************************************************************/
// square root of GMP_integer
// calculates z = squareroot(z1)
// added: 01.02.2001
// Last change: 01.02.2001
// Arguments: 1. GMP_integer
// 		  2. GMP_integer
//**************************************************************************/

void int_gmp_sqrt(int_gmp_type z1, int_gmp_type z)
{
 mpz_sqrt(z,z1);
 return;
}


//**************************************************************************/
// shifting GMP_integer by n bits
// calculates z = z1 <<(>>) z2
// added: 01.02.2001
// Last change: 01.02.2001
// Arguments: 1. GMP_integer
// 		  2. GMP_integer
// 		  3. int integer n
//**************************************************************************/

void int_gmp_shift(int_gmp_type z1, int_gmp_type z, int n)
{
  if (n>=0) mpz_mul_2exp(z, z1, n);
  else mpz_tdiv_q_2exp(z, z1, -n);  /** truncate-div **/
}

//**************************************************************************/
// sign and comparison of integer 
//**************************************************************************/

int int_gmp_sgn(const int_gmp_type z){return mpz_sgn(z);};
int int_gmp_cmp(const int_gmp_type z1,const  int_gmp_type z2){
      return mpz_cmp(z1,z2);};

==> iRRAM_2005_03.tar.gz: iRRAM_2005_03/src/GMP_int_ext.c <==
/*

GMP_int_ext.c -- extension to the GMP integer functions for the iRRAM library 
 
Copyright (C) 2001-2004 Norbert Mueller
 
This file is part of the iRRAM Library.
 
The iRRAM Library is free software; you can redistribute it and/or modify
it under the terms of the GNU Library General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.
 
The iRRAM Library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
License for more details.
 
You should have received a copy of the GNU Library General Public License
along with the iRRAM Library; see the file COPYING.LIB.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA. 
*/

#include "GMP_intrat.h"

#include <strings.h>

#ifndef DEBUG			// debugging information
#define DEBUG 0
#endif
#if DEBUG > 0
  int DEBUG_OP = 1;
  unsigned int op_count=0;
  int minshowop = DEBUG ;
#endif

//**************************************************************************/
//* Variables for counting free space */
//**************************************************************************/

int_gmp_type gmp_FreeVarsi[MaxFreeVars];
int gmp_FreeVarCounti=0L;
int int_gmp_var_count=0;


//**************************************************************************/
// nth root of integer in GMP
// added: 07.02.2001
// Last change: 07.02.2001
// Comment: this root-function truncates the result to the integer part
// Arguments: 1. GMP_integer
// 		  2. unsigned int
// 		  3. GMP_integer
//**************************************************************************/

void int_gmp_root(int_gmp_type z1, unsigned int z2, int_gmp_type z)
{
 int temp;	
 temp=mpz_root(z, z1, z2);
}

//**************************************************************************/
// power function in GMP
// calculates z = z1 ^ z2
// added: 07.02.2001
// Last change: 07.02.2001
// Comment: base=MP exponent=int
// Arguments: 1. GMP_integer
// 		  2. unsigned integer
// 		  3. GMP_integer
//**************************************************************************/

void int_gmp_power_i(int_gmp_type z1, unsigned int z2, int_gmp_type z)
{
 mpz_pow_ui(z, z1, z2);
 return;
}

//**************************************************************************/
// power function in GMP
// calculates z = z1 ^ z2
// added: 07.02.2001
// Last change: 07.02.2001
// Comment: base=int exponent=int
// Arguments: 1. unsigned int
// 		  2. unsigned int
// 		  3. GMP_integer
//**************************************************************************/

void int_gmp_power_ii(unsigned int z1, unsigned int z2, int_gmp_type z)
{
 mpz_ui_pow_ui(z, z1, z2);
 return;
}

//**************************************************************************/
// faculty function in GMP
// calculates z = z1!
// added: 07.02.2001
// Last change: 07.02.2001
// Comment: base is int
// Arguments: 1. unsigned int
// 		  2. GMP_integer
//**************************************************************************/

void int_gmp_fac(unsigned int z1, int_gmp_type z)
{
 mpz_fac_ui(z, z1);
 return;
}

//**************************************************************************/
// modulo function in GMP
// calculates z = z1 % z2
// added: 07.02.2001
// Last change: 07.02.2001
// Arguments: 1. GMP_integer
// 		  2. GMP_integer
// 		  3. GMP_integer
//**************************************************************************/

void int_gmp_modulo(int_gmp_type z1, int_gmp_type z2, int_gmp_type z)
{
 mpz_mod(z, z1, z2);
 return;
}





//**************************************************************************/
// writes integer to a string
// writes z to string with length w
// Arguments: 1. GMP_integer
// 		  2. int integer
//	return: char*
//**************************************************************************/


char* int_gmp_swritee(int_gmp_type z, int w)
{
  if (w<1)w=1;
  char *n, *s=malloc(w+1);
  int l;
  n=mpz_get_str(NULL, 10, z);
  l=strlen(n);
  if (l>w){
    strncpy(s,n,w);
    s[w-1]='*';
    s[w]='\0';
  }else{
    memset(s,' ',w-l);
    strncpy(&s[w-l],n,l);
    s[w]='\0';
  } 
  free(n); 
  return s;
}



//**************************************************************************/
// writes z to string with length calculated
// added: 18.07.2001
// Last change: 18.07.2001
// Arguments: 1. GMP_integer
// 	return: char*
//**************************************************************************/

char* int_gmp_sprintf(int_gmp_type z)
{
  return mpz_get_str(NULL,10,z);
}


//**************************************************************************/
// square root of GMP_integer
// calculates z = squareroot(z1)
// added: 01.02.2001
// Last change: 01.02.2001
// Arguments: 1. GMP_integer
// 		  2. GMP_integer
//**************************************************************************/

void int_gmp_sqrt(int_gmp_type z1, int_gmp_type z)
{
 mpz_sqrt(z,z1);
 return;
}


//**************************************************************************/
// shifting GMP_integer by n bits
// calculates z = z1 <<(>>) z2
// added: 01.02.2001
// Last change: 01.02.2001
// Arguments: 1. GMP_integer
// 		  2. GMP_integer
// 		  3. int integer n
//**************************************************************************/

void int_gmp_shift(int_gmp_type z1, int_gmp_type z, int n)
{
  if (n>=0) mpz_mul_2exp(z, z1, n);
  else mpz_tdiv_q_2exp(z, z1, -n);  /** truncate-div **/
}
